package net.ch2.pc8.imouto;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;

import net.ch2.pc8.imouto.person.Imouto;
import net.ch2.pc8.imouto.person.Onichan;
import net.ch2.pc8.imouto.person.Person;
import net.ch2.pc8.imouto.person.thread.ImoutoTalkThreadBase;
import net.ch2.pc8.imouto.person.thread.ImoutoWritingThread;
import net.ch2.pc8.imouto.ui.PopupMenu;
import net.ch2.pc8.imouto.ui.TaskTray;
import net.ch2.pc8.imouto.ui.shell.DesktopShell;
import net.ch2.pc8.imouto.ui.shell.TalkShell;
import net.ch2.pc8.imouto.util.Constant;
import net.ch2.pc8.imouto.util.Debugger;
import net.ch2.pc8.imouto.util.ImageFactory;
import net.ch2.pc8.imouto.util.MyProperties;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import com.asprise.books.javaui.ch06.BorderLayout;

public class Mediator {

    private TalkShell talkShell;
    private DesktopShell desktopShell;
    private TaskTray taskTray;
    private PopupMenu popupMenu;
    private Display display;
    private Onichan onichan;
    private Imouto imouto;
    
    private MyProperties properties;
    private String imoutoDirectoryName;
    private ArrayList threadList;
    
    public Mediator() {
        this.display = new Display();
        this.desktopShell = new DesktopShell(this);
        createTalkShell();
        this.onichan = new Onichan(this, "おにーちゃん", display.getSystemColor(SWT.COLOR_BLACK));
        this.imouto = new Imouto(this, "いもうと", display.getSystemColor(SWT.COLOR_RED));
        this.popupMenu = new PopupMenu(this);
        this.taskTray = new TaskTray(this);
        this.threadList = new ArrayList(3);
    }

    
    /**
     * 設定ファイルをロードする。
     * 
     */
    private void loadConfig() {
        FileInputStream fis = null;
        try {
            fis = new FileInputStream( Constant.FILE_CONFIG );            
            properties = new MyProperties();
            properties.load(fis);
            imoutoDirectoryName = properties.get(Constant.IMOUTO_DIRECTORY_NAME, getImoutoDirectories()[0].getName());
            setImoutoDirectory(imoutoDirectoryName);
            popupMenu.selectNowImoutoMenuItem(imoutoDirectoryName);
            int defaultX = desktopShell.getShell().getMonitor().getClientArea().width  - desktopShell.getImage().getImageData().width;
            int defaultY = desktopShell.getShell().getMonitor().getClientArea().height - desktopShell.getImage().getImageData().height;
            desktopShell.setLocation( properties.getInt(Constant.DESKTOP_SHELL_X_POSITION, defaultX), defaultY );
            talkShell.setLocation( properties.getInt(Constant.TALK_SHELL_X_POSITION, defaultX-talkShell.getShell().getSize().x), properties.getInt(Constant.TALK_SHELL_Y_POSITION, defaultY) );
        } catch( FileNotFoundException fnfe ) {
            try {
                if( new File( Constant.FILE_CONFIG ).createNewFile() ) {
                    loadConfig();
                } else { fnfe.printStackTrace(); }
            } catch( IOException ioe ) { 
                if( new File( Constant.DIRECTORY_CONFIG ).mkdir() ) {
                    loadConfig();
                } else { ioe.printStackTrace(); }
            }
        } catch( IOException fnfe ) {
            fnfe.printStackTrace();
        }
    }


    public void init() {
//        setImoutoFace( Imouto.FACE_SU );

        loadConfig();
//        getThreadModules();

        
        desktopShell.open();
        talkShell.open();
        imouto.greet();
        
        while (!desktopShell.isDisposed() ) {
            if (!display.readAndDispatch()) {
                display.sleep();
            }
        }
//        exit();
    }
    
    // TODO このへんダサいがしょうがないか？
    public Display getDisplay() { return this.display; }
    public Onichan getOnichan() { return this.onichan; } // TalkShell#keyPressed()で使ってる
    public ImoutoWritingThread getImoutoWritingThread() { return imouto.getImoutoWritingThread(); }
    public Imouto getImouto() { return this.imouto;}
    public String getImoutoDirectoryName() { return this.imoutoDirectoryName; }

    /**
     * 会話画面を作成する。
     * 
     */
    private void createTalkShell() {
        this.talkShell = new TalkShell(this);
        this.talkShell.setLayout(new BorderLayout());
        this.talkShell.setText("talk to my little sister");
        this.talkShell.setImage(new Image(null, Constant.ICON_TITLE));
    }

    /**
     * 会話画面を開く。 disposedされていれば新規作成。
     * 
     */
    public void openTalkShell() {
        if (talkShell.isDisposed()) {
            createTalkShell();
            talkShell.setLocation(properties.getInt(Constant.TALK_SHELL_X_POSITION, 0), properties.getInt(Constant.TALK_SHELL_Y_POSITION, 0));
        }
        setOnTopAndOpen();
    }

    /**
     * DesktopShellとTalkShellを最前面表示する。
     * 
     */
    public void setOnTopAndOpen() {
        getShellOfDesktopShell().forceActive();
        getShellOfTalkShell().forceActive();
        talkShell.open();
    }

    public void disposeTalkShell() {
        this.talkShell.dispose();
    }

    /**
     * ポップアップメニューを表示する。
     * 
     */
    public void displayPopupMenu() {
        popupMenu.setVisible(true);
    }

    public void talk(Person person, String message) {
        talk(person, message, null, true, true);
    }

    // /*public*/private void talkName(Person person) {
    // talkShell.appendTextArea( person.getColor(), person.getName() + "：" );
    // }

    public void talk(Person person, String message, String voiceFileName) {
        talk(person, message, voiceFileName, true, true);
    }
    
    public void talk(Person person, String message, boolean isStart, boolean doLineFeed) {
        talk(person, message, null, isStart, doLineFeed);
    }

    public void talk(Person person, String message, String voiceFileName, boolean isStart, boolean doLineFeed) {
        // Debugger.debug( person + ":" + message );
        // メッセージの初め→名前を出す。
        // if( isStart ) { talkName(person); }
        if( !isTalkShellOpen() ) openTalkShell();
        talkShell.appendTextArea(person.getColor(), (doLineFeed ? message + "\n" : message));
        if (person == onichan) {
            talkShell.clearInputText();
            imouto.kiduku(message.trim());
        } else if( voiceFileName != null && person == imouto ) {
            // TODO 未対応にしておく
//            MyUtil.playMP3(Constant.DIRECTORY_IMOUTO + "/" + this.imoutoDirectoryName + "/" + "voice/" + voiceFileName);
        }
    }

    public File[] getImoutoDirectories() {
        File[] listFiles = new File(Constant.DIRECTORY_IMOUTO).listFiles();
        ArrayList resultFiles = new ArrayList();
        for (int i = 0; i < listFiles.length; i++) {
            if (listFiles[i].isDirectory()) {
                resultFiles.add( listFiles[i] );
            }
        }
        return (File[])resultFiles.toArray(new File[0]);
    }

    /**
     * 設定が変わったことを受け取る。 この時点で設定ファイルに書き込みを行う。
     * 
     */
    public void notifyToChangeConfig() {
//      Debugger.debug( "設定が変わりました" );
        // TODO ダサい
        if( !talkShell.isDisposed() ) {
            properties.putInt( Constant.TALK_SHELL_X_POSITION, talkShell.getLocation().x );
            properties.putInt( Constant.TALK_SHELL_Y_POSITION, talkShell.getLocation().y );
        }
        if( !desktopShell.isDisposed() ) {
            properties.putInt( Constant.DESKTOP_SHELL_X_POSITION, desktopShell.getShell().getLocation().x );
        }
//        properties.putInt( Constant.DESKTOP_SHELL_Y_POSITION, desktopShell.getShell().getLocation().y );
        saveConfig();
    }

    /**
     * 設定ファイルを書き込む。
     * 
     */
    public void saveConfig() {
Debugger.debug( "設定ファイルに書き込みます。" );
        try {
            FileOutputStream fos = new FileOutputStream(Constant.FILE_CONFIG);
            properties.store(fos, "Configration Infomation");
            fos.flush();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * いもうとの表情を設定する。
     * 
     * @param faceID
     *            表情
     */
    public void setImoutoFace(String faceID) {
        ImageData image = ImageFactory.getImageData(Constant.DIRECTORY_IMOUTO + "/" + this.imoutoDirectoryName + "/" + "surface/surface" + faceID + ".png");
        desktopShell.setImage(image);
    }

    /**
     * DesktopShellの位置を変更する。
     * @param 画像の横幅
     * @param 画像の高さ
     */
    private void changeDesktopShellLocation(int width, int height) {
        desktopShell.setLocation(display.getClientArea().width-width, display.getClientArea().height-height);
    }
    
    public Shell getShellOfDesktopShell() {
        return desktopShell.getShell();
    }

    public Shell getShellOfTalkShell() {
        return talkShell.getShell();
    }

    /**
     * 終了処理
     * 
     */
    public void exit() {
        stopThreads();
        taskTray.dispose();
        talkShell.dispose();
        desktopShell.dispose();
        display.dispose();
    }

    /**
     * サイズ変更
     * 
     */
    public void changeDesktopSize() {
        // TODO 未実装
    }

    public boolean isDesktopShellDisposed() {
        return desktopShell.isDisposed();
    }

    public void setInputTextOfTalkShell(boolean b) {
        talkShell.setInputText(b);
    }

    public boolean isTalkShellOpen() {
        return talkShell.isOpen();
    }

    public void setImoutoDirectory(String imoutoDirectoryName) {
        if( imoutoDirectoryName == null ) {
            this.imoutoDirectoryName = getImoutoDirectories()[0].getName();
        } else {
            this.imoutoDirectoryName = imoutoDirectoryName;
        }
        setImoutoFace( Imouto.FACE_SU );
    }
    
    /**
     * いもうと変更
     *
     */
    public void changeImouto(String imoutoDirectoryName) {
        setImoutoDirectory(imoutoDirectoryName);
        properties.put( Constant.IMOUTO_DIRECTORY_NAME, imoutoDirectoryName );
        changeDesktopShellLocation( desktopShell.getImage().getBounds().width, desktopShell.getImage().getBounds().height );
        notifyToChangeConfig();    
    }
    
//    public String getImoutoDirectoryName() {
//        return Configuration.imoutoDirectoryName;
//    }

    // TODO start()で再実行できない
    public void startThread(ImoutoTalkThreadBase threadBase) {
        talk( imouto, threadBase.outline() );
//        display.syncExec( threadBase.new DisposeThread(5000) );
        threadList.add( threadBase );
        threadBase.start();
    }
    
    public void stopThread(ImoutoTalkThreadBase threadBase) {
        threadBase.interrupt();
        threadList.remove(threadBase);
    }
    
//    public void startThread(ImoutoTalkThreadBase threadBase) {
//        Thread th = new Thread(threadBase);
//        threadList.add( th );
//        th.start();
//    }
//    
//    public void stopThread(ImoutoTalkThreadBase threadBase) {
//        Thread th = new Thread(threadBase);
//        for( int i=0; i<threadList.size(); i++ ) {
//            Thread listThread = (Thread)threadList.get(i);
//            if( th.getName().equals(listThread.getName()) ) {
//                listThread.interrupt();
//                threadList.remove(listThread);
//            }
//        }
//    }
    
    public void stopThreads() {
        ImoutoTalkThreadBase[] threadBases = (ImoutoTalkThreadBase[])threadList.toArray(new ImoutoTalkThreadBase[0]);
        for( int i=0; i<threadBases.length; i++ ) {
            stopThread(threadBases[i]);
        }
        threadBases = null;
        threadList = null;
    }
}
