package net.ch2.pc8.imouto.person.thread;

import net.ch2.pc8.imouto.Mediator;
import net.ch2.pc8.imouto.util.Debugger;

public abstract class ImoutoTalkThreadBase extends Thread {
    protected Mediator mediator;
    protected OpenThread openThread;
    protected DisposeThread disposeThread;
    protected static long lastTalkTime;
    
    private ImoutoTalkThreadBase() {}
    protected ImoutoTalkThreadBase( Mediator mediator ) {
        this.mediator = mediator;
        this.openThread = new OpenThread();
        this.disposeThread = new DisposeThread();
    }

    // TODO 消極的な方法...
    protected void checkWindow() {
        long now = System.currentTimeMillis();
Debugger.debug( ""+(now-lastTalkTime) );        
        if( now - lastTalkTime > 20*1000 ) {
            mediator.getDisplay().syncExec(disposeThread);
        }
    }
    
    /**
     * 説明する。
     * @return 説明文
     */
    public abstract String outline();
  
    protected void talk(String faceID, String imoutoMessage) {
        talk(faceID, imoutoMessage, null, true, true);
    }

    protected void talk(String faceID, String imoutoMessage, String voiceFileName) {
        talk( faceID, imoutoMessage, voiceFileName, true, true );
    }
    
    protected void talk(String faceID, String imoutoMessage, boolean isStart, boolean doLineFeed) {
        talk( faceID, imoutoMessage, null, isStart, doLineFeed );
    }
    
    protected void talk(String faceID, String imoutoMessage, String voiceFileName, boolean isStart, boolean doLineFeed) {
        mediator.getDisplay().syncExec(new TalkThread(faceID, imoutoMessage, voiceFileName, isStart, doLineFeed));
    }
    
    protected void noWaitTalk( String faceID, String imoutoMessage ) {
        noWaitTalk(faceID, imoutoMessage, null, true, true);
    }
    
    protected void noWaitTalk( String faceID, String imoutoMessage, String voiceFileName ) {
        noWaitTalk(faceID, imoutoMessage, voiceFileName, true, true);
    }

    protected void noWaitTalk(String faceID, String imoutoMessage, boolean isStart, boolean doLineFeed) {
        noWaitTalk(faceID, imoutoMessage, null, isStart, doLineFeed);
    }
    
    protected void noWaitTalk( String faceID, String imoutoMessage, String voiceFileName, boolean isStart, boolean doLineFeed ) {
        NoWaitTalkThread noWaitTalkThread = new NoWaitTalkThread(faceID, imoutoMessage, voiceFileName, isStart, doLineFeed);
        mediator.getDisplay().syncExec( openThread );
        mediator.getDisplay().syncExec( noWaitTalkThread );
        noWaitTalkThread = null;
    }
    
    public class TalkThread implements Runnable {
        String faceID, imoutoMessage, voiceFileName;
        boolean isStart, doLineFeed;
        public TalkThread(String faceID, String imoutoMessage, String voiceFileName, boolean isStart, boolean doLineFeed) {
            this.faceID = faceID;
            this.imoutoMessage = imoutoMessage;
            this.voiceFileName = voiceFileName;
            this.isStart = isStart;
            this.doLineFeed = doLineFeed;
        }
        public void run() {
            // 入力しているっぽい雰囲気
//imouto.getMediator().setInputTextOfTalkShell(false);
            try {
                Thread.sleep( imoutoMessage.length()*100 );
            } catch( InterruptedException ie ) {}
//imouto.getMediator().setInputTextOfTalkShell(true);
            
            mediator.getImouto().talk( faceID, imoutoMessage, voiceFileName, isStart, doLineFeed );
        }        
    }

    public class ExitThread implements Runnable {
        public void run() {
            try {
                Thread.sleep( 1500 );
            } catch( InterruptedException ie ) {}
            mediator.exit();
        }
    }
    
    public class NoWaitTalkThread implements Runnable {
        String faceID, imoutoMessage, voiceFileName;
        boolean isStart, doLineFeed;
        NoWaitTalkThread(String faceID, String imoutoMessage, String voiceFileName, boolean isStart, boolean doLineFeed) {
            this.faceID = faceID;
            this.imoutoMessage = imoutoMessage;
            this.voiceFileName = voiceFileName;
            this.isStart = isStart;
            this.doLineFeed = doLineFeed;
        }
        public void run() {
            mediator.getImouto().talk( faceID, imoutoMessage, voiceFileName, isStart, doLineFeed );            
        }        
    }
    
    public class OpenThread implements Runnable {
        public void run() {
            mediator.openTalkShell();
        }
    }
    
    public class DisposeThread implements Runnable {
        int millisec;
        public DisposeThread() {}
        public DisposeThread(int millisec) {
            this.millisec = millisec;
        }
        public void run() {
            long start = System.currentTimeMillis();
            while( System.currentTimeMillis() - start <= millisec ) ;
            mediator.disposeTalkShell();
        }
    }
    
    public boolean equals(Object o) {
        if( !(o instanceof ImoutoTalkThreadBase) ) return false;
        ImoutoTalkThreadBase base = (ImoutoTalkThreadBase)o;
//Debugger.debug( base.getName() + ", " + this.getName() );        
        if( base.getName().equals(this.getName()) ) return true;
        else return false;
    }
    
    public int hashCode() {
        return getName().hashCode();
    }

}
