package net.ch2.pc8.imouto.person.thread.user;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;

import net.ch2.pc8.imouto.Mediator;
import net.ch2.pc8.imouto.person.Imouto;
import net.ch2.pc8.imouto.person.thread.ImoutoTalkThreadBase;
import net.ch2.pc8.imouto.util.Constant;
import net.ch2.pc8.imouto.util.Debugger;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ResIterator;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.util.FileManager;
import com.hp.hpl.jena.vocabulary.RDF;
import com.hp.hpl.jena.vocabulary.RSS;

/**
 * サンプル実装 - RSSリーダー
 * @author ore
 *
 */
public class ImoutoRSSTalkThread extends ImoutoTalkThreadBase {
//    private long lastTalkTime;
//    private boolean openForNewsTalk;
//    private boolean stop;
//    private String[] newsURLs;
    private long lastTalkTimeThis;

    public ImoutoRSSTalkThread(Mediator mediator) {
        super(mediator);
        setName( "RSSリーダー" );
    }
    
    /**
     * rssファイルからRSSサイトのURLリストを取得し返却。
     * @return
     */
    private String[] createRSSURLs() {
        ArrayList newsList = new ArrayList();
        String[] rssURLs = null;
        try {
            BufferedReader br = new BufferedReader(new FileReader(Constant.DIRECTORY_IMOUTO + mediator.getImoutoDirectoryName() + "/" + Constant.FILE_RSS_LIST));
            for( String line; (line=br.readLine()) != null; ) {
                // #(シャープ)で始まる行はコメント行にする
                if( line.charAt(0) == '#' ) continue;
                newsList.add(line);
            }
            Collections.shuffle(newsList);
            rssURLs = (String[])newsList.toArray(new String[0]);
        } catch( FileNotFoundException fnfe ) {
            fnfe.printStackTrace();
        } catch( IOException ioe ) {
            ioe.printStackTrace();
        }
        return rssURLs;
    }
    
    /**
     * RSSサイトにアクセスしてResourceを取得する。
     * ネットワークアクセスあり!!
     * @param RSSサイトURLリスト
     * @return RSSのResource[]
     */
    private Resource[] getResources( String[] rssURLs ) {
        ArrayList resourceList = new ArrayList();
        for( int i=0; i<rssURLs.length; i++ ) {
            String url = rssURLs[i];
            Model model = FileManager.get().loadModel(url);
            ResIterator resIterator = model.listSubjectsWithProperty( RDF.type, RSS.item );
            while( resIterator.hasNext() ) {
                resourceList.add( resIterator.nextResource() );
            }
        }
        Collections.shuffle(resourceList);
        return (Resource[])resourceList.toArray(new Resource[0]);
    }
    
    
    
    public void run() {
        lastTalkTime = System.currentTimeMillis();
        lastTalkTimeThis = lastTalkTime;
//        openForNewsTalk = true;
        String[] rssURLs = createRSSURLs();
        try {
            while( true ) {
                Resource[] resources = getResources( rssURLs );
//Debugger.debug( resources );                
                for( int i=0; i<resources.length;) {
                    long now = System.currentTimeMillis();
Debugger.debug( "差: "+(now-lastTalkTime) );
Debugger.debug( "lTT: "+lastTalkTime );
                    if( now-lastTalkTimeThis > 240*1000 /*&& !mediator.isTalkShellOpen()*/ ) {
                        Statement title = resources[i].getProperty(RSS.title);
                        Statement link  = resources[i].getProperty(RSS.link);
                        noWaitTalk( Imouto.FACE_SU, "ねぇねぇ、" +  title.getString() + " だって。\n" + link.getString() );
                        lastTalkTime = now;
                        lastTalkTimeThis = lastTalkTime;
//                        openForNewsTalk = true;
                        i++;
//                    } else if( now-lastTalkTime > 30*1000 && openForNewsTalk ) {
//                        mediator.getDisplay().syncExec( disposeThread );
//                        openForNewsTalk = false;
                    }
                    sleep(1000);
                    checkWindow();
                }
            }
        } catch( InterruptedException ie ) {
        }

Debugger.debug( "スレッド" + getName() + "が終了します..." );        
    }
    
    public String outline() {
        return "インターネットで見たニュースをおにーちゃんに知らせてあげるよ。";
    }

}
