package net.ch2.pc8.imouto.ui;

import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Shell;

import com.asprise.books.javaui.ch06.BorderLayout.BorderData;

/**
 * 
 * @see http://cjasmin.fc2web.com/samples/text_hyperlink.html
 *
 */
public class MyStyledText implements MouseListener, MouseMoveListener {
    private StyledText styledText;
    private Vector ranges = new Vector();
    private Cursor handCursor, arrowCursor;
//    private Shell shell;

    public MyStyledText(Shell shell, int i) {
        styledText = new StyledText(shell, i);
        handCursor = new Cursor(shell.getDisplay(), SWT.CURSOR_HAND);
        arrowCursor = new Cursor(shell.getDisplay(), SWT.CURSOR_ARROW);
        styledText.addMouseListener(this);
        styledText.addMouseMoveListener(this);
    }

    public void setEditable(boolean b) {
        styledText.setEditable(b);
    }

    public void setLayoutData(BorderData data) {
        styledText.setLayoutData(data);
    }

    public void addModifyListener(ModifyListener listener) {
        styledText.addModifyListener(listener);
    }

    public String getText() {
        return styledText.getText();
    }

    public void setCaretOffset(int i) {
        styledText.setCaretOffset(i);
    }

    public void showSelection() {
        styledText.showSelection();
    }

    public void append(String message) {
        styledText.append(message);
    }

    public void setStyleRanges(StyleRange[] ranges) {
        styledText.setStyleRanges(ranges);
    }

    private void createHyperLink(StyledText text) {
        String content = text.getText();
        Pattern pattern = Pattern.compile("http://[a-zA-Z\\./\\?&\\-=:_0-9%]*");
        Matcher matcher = pattern.matcher(content);

        while (matcher.find()) {
            String line = matcher.group();
            int index = content.indexOf(line);
            HyperLinkRange range = new HyperLinkRange();
            range.start = index;
            range.length = line.length();
            range.underline = true;
            range.foreground = styledText.getShell().getDisplay().getSystemColor(SWT.COLOR_BLUE);
            range.keyword = line;
            ranges.add(range);

            text.setStyleRange(range);
        }
    }

    // カーソル上にハイパーリンクがあればHyperLinkRangeを返し、
    // それ以外はnullを返す。
    private HyperLinkRange getHyperLinkRange(int x, int y) {
        Point p = new Point(x, y);
        try {
            int offset = styledText.getOffsetAtLocation(p);
            for (int i = 0; i < ranges.size(); i++) {
                HyperLinkRange r = (HyperLinkRange) ranges.get(i);
                int offset1 = r.start;
                int offset2 = offset1 + r.length;
                if (offset >= offset1 && offset <= offset2) {
                    return r;
                }
            }
        } catch (IllegalArgumentException ex) {
            // カーソルの位置によっては、Offsetが計算できず、
            // Exceptionが発生する。したがってここでは何もしない。
        }
        return null;
    }

    public void mouseDoubleClick(MouseEvent e) {
    }

    public void mouseDown(MouseEvent event) {
        // 左クリックのとき
        if( event.button == 1 ) {
            int x = event.x;
            int y = event.y;
            HyperLinkRange range = getHyperLinkRange(x, y);
            if (range != null) {
                Program program = Program.findProgram("html");
                program.execute(range.keyword);
            }
        }
    }

    public void mouseUp(MouseEvent e) {
    }

    public void mouseMove(MouseEvent event) {
        int x = event.x;
        int y = event.y;
        HyperLinkRange range = getHyperLinkRange(x, y);
        if (range != null) {
            styledText.setCursor(handCursor);
        } else {
            styledText.setCursor(arrowCursor);
        }
    }

    class HyperLinkRange extends StyleRange {
        String keyword;
    }

    public void createHyperLink() {
        createHyperLink(styledText);
    }

    public int getLineCount() {
        return styledText.getLineCount();
    }

    public void setTopIndex(int top) {
        styledText.setTopIndex(top);
    }
}
