package net.ch2.pc8.imouto.ui.shell;

import net.ch2.pc8.imouto.Mediator;
import net.ch2.pc8.imouto.util.Constant;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Region;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

/**
 * 
 * @see http://pc8.2ch.net/test/read.cgi/tech/1144500384/526n
 *
 */
public class DesktopShell implements Listener, KeyListener, MouseListener, MouseMoveListener, PaintListener {
    private Mediator mediator;
	private Shell shell;
	private Region region;
	private Image image;
	private Point origin, originLocation;
    private boolean isMoved;
	
	public DesktopShell(Mediator mediator) {
        this.mediator = mediator;
		shell = new Shell(mediator.getDisplay(), SWT.NO_TRIM /*| SWT.ON_TOP*/ | SWT.NO_BACKGROUND );
        shell.setText("偽");
        shell.setImage( new Image(null, new ImageData(Constant.ICON_TITLE)) );
        shell.setLayout( new GridLayout() );
        shell.addKeyListener(this);
        shell.addMouseListener(this);
        shell.addMouseMoveListener(this);
		shell.addPaintListener(this);
        shell.addListener(SWT.MenuDetect, this);
	}
    
    public void setLocation( int x, int y ) {
        shell.setLocation( x, y );
    }
    
    public Shell getShell() { return this.shell; }
    public Image getImage() { return this.image; }

	public void open() {
	    shell.open();
	}

	public boolean isDisposed() {
		return shell.isDisposed();
	}

	public void dispose() {
		shell.dispose();
		if(region!=null) region.dispose();
	}
    
    public void forceActive() {
        shell.forceActive();
    }

	public void setImage(ImageData data) {
		this.image = new Image(mediator.getDisplay(), data);
        region = new Region();
		final int mask = data.getPixel(0, 0);
        for (int x = 0; x < data.width; x++) {
			int start = 0;
			for (int y = 0; y < data.height; y++) {
				if (data.getPixel(x, y) == mask) {
					if (start != y) {
						region.add(x, start, 1, y - start);
					}
					start = y + 1;
				}
			}
			if (data.getPixel(x, data.height - 1) != mask) {
				region.add(x, start, 1, data.height - start);
			}
		}
		
		// リージョン設定
		shell.setSize(data.width, data.height);
		shell.setRegion(region);
        // 再描画
        shell.redraw();
	}

	public void paintControl(PaintEvent e) {
        e.gc.drawImage(image, 0, 0);
    }

    
    public void mouseDoubleClick(MouseEvent e) {
        // e.button 1番は左クリック
        if( e.button == 1 ) {
            mediator.openTalkShell();
        }
    }

    public void mouseDown(MouseEvent e) {
        if( e.button == 1 ) {
            origin = new Point(e.x, e.y);
            originLocation = shell.getShell().getLocation();
        }
    }

    public void mouseUp(MouseEvent e) {
        if( e.button == 1 && isMoved ) {
            mediator.notifyToChangeConfig();
        }
        origin = null;
        isMoved = false;
    }

    public void mouseMove(MouseEvent e) {        
        if( origin != null ) {
            Point p = mediator.getDisplay().map(shell.getShell(), null, e.x, e.y);
            shell.setLocation(p.x - origin.x, originLocation.y);
            isMoved = true;
        }
    }

    public void handleEvent(Event event) {
        if( event.type == SWT.MenuDetect ) {
            mediator.displayPopupMenu();
        }
    }

    public void keyPressed(KeyEvent e) {
        // ESCキーで元の位置に戻す。
        if( e.stateMask == SWT.BUTTON1 && e.keyCode == 27 ) {
            shell.setLocation( originLocation.x, originLocation.y );
            origin = null;
            isMoved = false;
        }
        // Ctrl-w
        if( e.stateMask == SWT.CTRL && e.keyCode == 119 ) {
            mediator.exit();
        }
    }

    public void keyReleased(KeyEvent e) {
        // nop
    }

    public Point getLocation() {
        return shell.getLocation();
    }
}
