package net.ch2.pc8.imouto.ui.shell;
import java.util.ArrayList;

import net.ch2.pc8.imouto.Mediator;
import net.ch2.pc8.imouto.person.Imouto;
import net.ch2.pc8.imouto.ui.MyStyledText;
import net.ch2.pc8.imouto.util.Debugger;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.asprise.books.javaui.ch06.BorderLayout;

public class TalkShell implements Listener, KeyListener, ModifyListener {

    private Mediator mediator;
    private Shell shell;
    private MyStyledText textArea;
    private ArrayList rangesList;
    private Text inputText;

    private int textCount;

    public TalkShell(Mediator mediator) {
        this.mediator = mediator;
        this.shell = new Shell( this.mediator.getDisplay(), SWT.DIALOG_TRIM );
        this.shell.addListener( SWT.Move, this );
        this.shell.addListener( SWT.Dispose, this );
        this.shell.setSize( 300, 200 );
        
        textArea = new MyStyledText(shell, SWT.MULTI /*| SWT.BORDER | SWT.H_SCROLL */| SWT.V_SCROLL | SWT.WRAP);
        textArea.setEditable(false);
        textArea.setLayoutData(new BorderLayout.BorderData(BorderLayout.CENTER));
        textArea.addModifyListener(this);
        inputText = new Text(shell, SWT.SINGLE | SWT.BORDER);
        inputText.setTextLimit(100);
        inputText.setLayoutData(new BorderLayout.BorderData(BorderLayout.SOUTH));
        inputText.addKeyListener(this);
        inputText.setText("Ctrl-Enterで話しかけてね。");
        // TODO IME制御
//      inputText.addFocusListener(new FocusAdapter() {
//          public void focusGained(FocusEvent e) {
//Debugger.debug( "focusGained" );
//              talkShell.setImeInputMode(SWT.ROMAN);
//          }
//      });
        inputText.setFocus();
        inputText.selectAll();
    }

    public Shell getShell() { return shell; }

    public void setLayout(BorderLayout layout) {
        shell.setLayout(layout);
    }


    public void setText(String string) {
        shell.setText(string);
    }


    public void setImage(Image image) {
        shell.setImage(image);
    }


    public void open() {
        shell.open();
        // TODO 気づかせるとか
    }

    public void setLocation(int x, int y) {
        shell.setLocation( x, y );
    }


    public Point getLocation() {
        return shell.getLocation();
    }


    public void handleEvent(Event event) {
Debugger.debug(event);
        switch( event.type ) {
            case SWT.Move: {        
                mediator.notifyToChangeConfig();
                break;
            }
            case SWT.Dispose: {
                if( !mediator.isDesktopShellDisposed() ) {
                    mediator.setImoutoFace( Imouto.FACE_SU );
                    break;
                }
            }
        }
    }

    public void setImeInputMode(int mode) {
        shell.setImeInputMode(mode);
    }
    
    public boolean isDisposed() {
        return shell.isDisposed();
    }
    
    public boolean isOpen() {
//Debugger.debug(""+shell.isEnabled());
        // TODO TalkShellが開いている == 「disposedではない」、は嘘。一回も開いたことがないときが抜けてる。
        return !isDisposed();
    }

    public void dispose() {
        shell.dispose();
        textCount = 0;
        rangesList = null;
        // TODO
        System.gc();
    }  
    
    
    public void keyPressed(KeyEvent ke) {
        switch( ke.stateMask ) {
        // Ctrlボタンが押された状態
        case SWT.CTRL: {
            switch( ke.keyCode ) {
            // Ctrl-w
            case 119: {
                shell.dispose();
                break;
            // Ctrl-Enter
            } case 13: {
                if( inputText.getText().length() != 0 ) {
                    mediator.talk(mediator.getOnichan(), inputText.getText(), true, true);
                }
                break;
            }
            }
            break;
        }
        }
        switch( ke.keyCode ) {
        // ESC
        case 27: {
            shell.dispose();
            break;
        }
        }
    }

    public void keyReleased(KeyEvent ke) {
        // nop
    }

    public void modifyText(ModifyEvent me) {
//        // TODO 処理重げ
//        // キャレットを一番下にして
        textArea.setCaretOffset( textArea.getText().length() );
//        // そこにスクロールする。
        textArea.showSelection();
//        textArea.setTopIndex( textArea.getLineCount() );
    }

    /**
     * 会話の内容をメッセージ欄に追加する。
     * @param color   色
     * @param message 追加する言葉
     */
    public void appendTextArea(Color color, String message) {
        if( rangesList == null ) { rangesList = new ArrayList(); }
        StyleRange range = new StyleRange();
        textArea.append( message );
        range.start = textCount;
        int length = message.length();
        textCount += length;
        range.length = length;
        range.foreground = color;
        rangesList.add(range);
        textArea.setStyleRanges((StyleRange[])rangesList.toArray(new StyleRange[0]));
        textArea.createHyperLink();
        range = null;
    }
    
    /**
     * 入力欄を消去する。
     *
     */
    public void clearInputText() {
        inputText.setText("");
    }
    
    public void setInputText(boolean b) {
        inputText.setEnabled(b);
    }

}
